// toggle an APD Schema Task element open or closed.
// DITA elements may need a different handler
function toggleAPDSchemaTask (taskNameElem) {
  var taskBodyID = taskNameElem.getAttribute('aria-controls'),
      taskBodyElem = document.getElementById(taskBodyID),
      shouldClose = (taskNameElem.getAttribute('aria-expanded') === "true"),
      maxAnimationHeight = window.innerHeight;

  if (shouldClose) { // when closing, we want the animation to start as quickly as possible for small tasks
    maxAnimationHeight = Math.min(window.innerHeight, taskBodyElem.clientHeight);
  }

  taskBodyElem.style.maxHeight = maxAnimationHeight + "px";
  taskBodyElem.addEventListener("webkitTransitionEnd", function () {
    taskBodyElem.style.maxHeight = "10000px"; // effectively infinite
  }, false);

  taskNameElem.setAttribute("aria-expanded", (! shouldClose));
  taskBodyElem.setAttribute("aria-hidden", shouldClose);
}

(function () {
  // add in a task disclosure triangle, which can be animated (bg images can't)
  var tasks = document.querySelectorAll(".Task");
  for (var i = 0; i < tasks.length; i++) {
    var task = tasks[i];

    // all tasks are open by default. if there is more than one task on the page, close them all
    // select the task titles, since that is what would be clicked to toggle
    var taskTitle = task.querySelector(".Name");
    if (taskTitle && tasks.length > 1) {
        toggleAPDSchemaTask(taskTitle);
    }
  }
}());

// A global variable to store the number of occurrences
// This helper function recursively searches in elements and their child nodes.
function highlightAllOccurencesOfStringForElement(element,terms) {

    var foundTermCount = 0;
    if (element && terms) {
        if (element.nodeType == 3) {        // Text node
            var hasMatch = false;
            for (var x = 0; x < terms.length; ++x) {
                var term = terms[x].toLowerCase();
                while (true) {
                    var value = element.nodeValue;  // Search for term in text node
                    var idx = value.toLowerCase().indexOf(term);

                    if (idx < 0) break;             // not found, abort

                    var span = document.createElement("span");
                    var text = document.createTextNode(value.substr(idx,term.length));
                    span.appendChild(text);
                    span.setAttribute("class","FoundTermHighlight");
                    span.style.backgroundColor      = "f820391";
                    span.style.backgroundImage      = "-webkit-linear-gradient(top, #fdefa1, #f2e07d)";
                    span.style.webkitBorderRadius   = "0.225em";
                    span.style.paddingTop           = "-0.4em";
                    span.style.paddingLeft          = "0.075em";
                    span.style.paddingRight         = "0.075em";
                    span.style.paddingBottom        = "0.2em";
                    span.style.color                = "#353535";
                    text = document.createTextNode(value.substr(idx+term.length));
                    element.deleteData(idx, value.length - idx);
                    var next = element.nextSibling;
                    element.parentNode.insertBefore(span, next);
                    element.parentNode.insertBefore(text, next);
                    element = text;
                    foundTermCount++;	// update the counter
                    hasMatch = true;
                }
            }

            if (hasMatch) {
                var elementParent = element.parentNode;
                while (elementParent) {
                    var taskCloseString = "Task closed";
                    var className = elementParent.className;
                    if (className == "Name" || (className == taskCloseString)) {
                        var shouldToggle = false;
                        if (className == taskCloseString) {
                            var nameElement = elementParent.querySelector(".Name");
                            if (nameElement) {
                                elementParent = nameElement;
                                shouldToggle = true;
                            }
                        }
                        else if (elementParent.parentNode.className == taskCloseString) {
                            shouldToggle = true;
                        }

                        if (shouldToggle) {
                            toggleAPDSchemaTask (elementParent);
                            break;
                        }
                    }
                    elementParent = elementParent.parentNode;
                }
            }
        }
        else if (element.nodeType == 1) { // Element node
            if (element.style.display != "none" && element.nodeName.toLowerCase() != 'select') {
                for (var i=element.childNodes.length-1; i>=0; i--) {
                    foundTermCount += highlightAllOccurencesOfStringForElement(element.childNodes[i],terms);
                }
            }
        }
    }
    return foundTermCount;
}

// Entry point to start the search
function highlightAllOccurencesOfString(terms) {
    return highlightAllOccurencesOfStringForElement(document.body, terms);
}

// This helper function recursively removes the highlights in elements and their children.
function removeAllHighlightsForElement(element) {
    if (element) {
        if (element.nodeType == 1) {
            if (element.getAttribute("class") == "FoundTermHighlight") {
                var text = element.removeChild(element.firstChild);
                element.parentNode.insertBefore(text,element);
                element.parentNode.removeChild(element);
                return true;
            } else {
                var normalize = false;
                for (var i=element.childNodes.length-1; i>=0; i--) {
                    if (removeAllHighlightsForElement(element.childNodes[i])) {
                        normalize = true;
                    }
                }
                if (normalize) {
                    element.normalize();
                }
            }
        }
    }
    return false;
}

// Entry point to remove the highlights
function removeAllHighlights() {
    removeAllHighlightsForElement(document.body);
}

// set up the open/close handler for Task elements
document.body.addEventListener("touchstart", function (e) {
  if (e.target.className.match(/Name/) && e.target.parentNode.className.match(/Task/)) {
    // we don't want to open/close tasks when the user taps and drags on the name
    var task = e.target,
        touchedAndReleased = true,
        // if the touch moves, this is not a tap
        moveHandler = function () { touchedAndReleased = false; },
        endHandler = function () {
          // if the touch did not move, then it was a tap: toggle the task
          if (touchedAndReleased) {
            toggleAPDSchemaTask(task);
          }

          // whether or not we toggled the task, stop listening to further touch events
          task.removeEventListener("touchmove", moveHandler);
          task.removeEventListener("touchend", endHandler);
        };

    // after initial touchstart, listen for touchmove and touchend events on the target object
    task.addEventListener("touchmove", moveHandler);
    task.addEventListener("touchend", endHandler);
  }
});

// DEBUG for desktop browser Task CSS checking
// document.body.addEventListener("click", function (e) {
//   if (e.target.className.match(/Name/) && e.target.parentNode.className.match(/Task/)) {
//     // we don't want to open/close tasks when the user taps and drags on the name
//     var task = e.target;
// 	toggleAPDSchemaTask(task);
//   }
// });

// set up an error handler for img tags
// the handler is attached to the body, but acts upon the errors that bubble up from img elements
document.body.addEventListener("error", function (e) {
  if (! e.target.nodeName.match(/img/i)) {
    return;
  }
  var errorSpan = document.createElement('span');
  errorSpan.appendChild(document.createTextNode(' [ ' + e.target.alt + ' ] '));
  e.target.parentNode.replaceChild(errorSpan, e.target);
});

// set a class on the body enabling animations. This prevents e.g. Task animation from firing on initial page load.
setTimeout(function() {
  document.body.className += " enableAnimation";
}, 50);
